//+------------------------------------------------------------------+
//|                                       AllWilder'sDMI_v1 600+.mq4 |
//|                                Copyright  2014, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
// List of MAs:
// MA_Method= 0: SMA        - Simple Moving Average
// MA_Method= 1: EMA        - Exponential Moving Average
// MA_Method= 2: Wilder     - Wilder Exponential Moving Average
// MA_Method= 3: LWMA       - Linear Weighted Moving Average 
// MA_Method= 4: SineWMA    - Sine Weighted Moving Average
// MA_Method= 5: TriMA      - Triangular Moving Average
// MA_Method= 6: LSMA       - Least Square Moving Average (or EPMA, Linear Regression Line)
// MA_Method= 7: SMMA       - Smoothed Moving Average
// MA_Method= 8: HMA        - Hull Moving Average by Alan Hull
// MA_Method= 9: ZeroLagEMA - Zero-Lag Exponential Moving Average
// MA_Method=10: DEMA       - Double Exponential Moving Average by Patrick Mulloy
// MA_Method=11: T3_basic   - T3 by T.Tillson (original version)
// MA_Method=12: ITrend     - Instantaneous Trendline by J.Ehlers
// MA_Method=13: Median     - Moving Median
// MA_Method=14: GeoMean    - Geometric Mean
// MA_Method=15: REMA       - Regularized EMA by Chris Satchwell
// MA_Method=16: ILRS       - Integral of Linear Regression Slope 
// MA_Method=17: IE/2       - Combination of LSMA and ILRS 
// MA_Method=18: TriMAgen   - Triangular Moving Average generalized by J.Ehlers
// MA_Method=19: VWMA       - Volume Weighted Moving Average 
// MA_Method=20: JSmooth    - Smoothing by Mark Jurik
// MA_Method=21: SMA_eq     - Simplified SMA
// MA_Method=22: ALMA       - Arnaud Legoux Moving Average
// MA_Method=23: TEMA       - Triple Exponential Moving Average by Patrick Mulloy
// MA_Method=24: T3         - T3 by T.Tillson (correct version)
// MA_Method=25: Laguerre   - Laguerre filter by J.Ehlers

// List of Prices:
// Price    = 0 - Close  
// Price    = 1 - Open  
// Price    = 2 - High  
// Price    = 3 - Low  
// Price    = 4 - Median Price   = (High+Low)/2  
// Price    = 5 - Typical Price  = (High+Low+Close)/3  
// Price    = 6 - Weighted Close = (High+Low+Close*2)/4

 
#property copyright "Copyright  2014, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_separate_window
#property indicator_buffers   4

#property indicator_color1    MediumSeaGreen
#property indicator_width1    1
#property indicator_color2    Tomato
#property indicator_width2    1
#property indicator_color3    DarkKhaki
#property indicator_width3    2 
#property indicator_color4    Pink
#property indicator_width4    1
#property indicator_style4    2 

//---- indicator parameters
extern int     TimeFrame         =     0;    //TimeFrame in min
extern int     MathMode          =     2;    //Math method: 0-RSI;1-Stoch;2-DMI
extern int     Price             =     0;    //Price = 0...10 (see List of Prices)
extern int     Length            =    14;    //Period of evaluation
extern int     ADX_Length        =    14;    //ADX period(0-off)
extern int     ADXR_Length       =    14;    //ADXR period(0-off)
extern int     PreSmooth         =     1;    //Pre-Smoothing Period 
extern int     Smooth            =     1;    //Smoothing period
extern int     MA_Method         =     0;    //MA Method(0...25)
extern int     VisualMode        =     0;    //0-typical,1-histogram
extern bool   alertsOn         = true;
extern bool   alertsOnCurrent  = false;
extern bool   alertsMessage    = true;
extern bool   alertsSound      = false;
extern bool   alertsEmail      = false;
extern bool   ShowArrows       = true;
extern string arrowsIdentifier = "DMI Arrows1";
extern double arrowsUpperGap   = 1.0;
extern double arrowsLowerGap   = 1.0;
extern color  arrowsUpColor    = LimeGreen;
extern color  arrowsDnColor    = Red;
extern int    arrowsUpCode     = 241;
extern int    arrowsDnCode     = 242;



//---- indicator buffers
double ADX[];
double ADXR[];
double PDI[];
double MDI[];
double iprice[];
double loprice[];
double bulls[];
double bears[];
double TR[];
double rPDI[];
double rMDI[];
double DX[];
double trend[];
//----
double tmp[][8][2], ma[8][3], mMA[2], mLo[2];
int    draw_begin, masize;
string TF, IndicatorName, short_name, math_name, maname;
datetime prevtime[8], prevditime;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   if(TimeFrame <= Period()) TimeFrame = Period();
   TF = tf(TimeFrame);
   if(TF == "N/A") TimeFrame = Period();
   
   IndicatorDigits(3);

//---- indicator buffers mapping
   IndicatorBuffers(13);
   SetIndexBuffer( 0,    PDI);
   SetIndexBuffer( 1,    MDI);
   SetIndexBuffer( 2,    ADX); SetIndexStyle(2,DRAW_LINE);
   SetIndexBuffer( 3,   ADXR); SetIndexStyle(3,DRAW_LINE);
   
   SetIndexBuffer( 4, iprice);
   SetIndexBuffer( 5,loprice);
   SetIndexBuffer( 6,  bulls);
   SetIndexBuffer( 7,  bears);   
   SetIndexBuffer( 8,     TR);
   SetIndexBuffer( 9,   rPDI);    
   SetIndexBuffer(10,   rMDI);
   SetIndexBuffer(11,     DX);
   SetIndexBuffer(12,     trend);
//---- drawing settings
   if(VisualMode == 0)
   {
   SetIndexStyle(0,DRAW_LINE);
   SetIndexStyle(1,DRAW_LINE);
   }
   else
   {
   SetIndexStyle(0,DRAW_HISTOGRAM,STYLE_SOLID,2);
   SetIndexStyle(1,DRAW_HISTOGRAM,STYLE_SOLID,2);
   }
   
   
   draw_begin = Bars - iBars(NULL,TimeFrame)*TimeFrame/Period() + Length + PreSmooth + Smooth + 1;
//---- indicator name
   switch(MathMode)
   {
   case 0 : math_name = "RSI"  ; break;
   case 1 : math_name = "Stoch"; break;
   case 2 : math_name = "DMI"  ; break;
   }
     
   maname = averageName(MA_Method,masize);
   
   IndicatorName = WindowExpertName(); 
   short_name = IndicatorName + "[" + TF + "]("+ math_name + "," + Price + "," + Length + "," + PreSmooth + "," + Smooth + "," + maname +")";
   IndicatorShortName(short_name);
   
   SetIndexDrawBegin(0,draw_begin);
   SetIndexDrawBegin(1,draw_begin);
   SetIndexDrawBegin(2,draw_begin + ADX_Length );
   SetIndexDrawBegin(3,draw_begin + ADXR_Length);
     
   
   SetIndexLabel(0,"+DI" );
   SetIndexLabel(1,"-DI" );     
   SetIndexLabel(2,"ADX" );
   SetIndexLabel(3,"ADXR");

   
   SetIndexEmptyValue(2,0);
   SetIndexEmptyValue(3,0);
//----
   ArrayResize(tmp,masize);
   
//---- initialization done
   
   return(0);
}

int deinit() 
{
   Comment(""); 
   deleteArrows();
   
return(0);
}

//+------------------------------------------------------------------+
//| AllWilder'sDMI_v1 600+                                           |
//+------------------------------------------------------------------+
int start()
{
   int limit, i, shift, counted_bars = IndicatorCounted(); 
   
   
   if(counted_bars > 0) limit = Bars - counted_bars - 1;
   if(counted_bars < 0) return(0);
   if(counted_bars < 1)
   { 
   limit = Bars-1;   
      for(i=limit;i>=0;i--) 
      {
      
      PDI[i]  = EMPTY_VALUE; 
      MDI[i]  = EMPTY_VALUE; 
      ADX[i]  = 0; 
      ADXR[i] = 0;
      }
   
      if(VisualMode == 0) SetLevelValue(0,20);
      else
      {
      SetLevelValue(0,  0);
      SetLevelValue(1, 10);
      SetLevelValue(2,-10);
      }
   SetLevelStyle(STYLE_DOT,0,DimGray);
   }   
   
   if(TimeFrame != Period())
	{
   limit = MathMax(limit,TimeFrame/Period());   
      
      for(shift = 0;shift < limit;shift++) 
      {	
      int y = iBarShift(NULL,TimeFrame,Time[shift]);
      
      
      
      PDI[shift]  = iCustom(NULL,TimeFrame,IndicatorName,0,MathMode,Price,Length,ADX_Length,ADXR_Length,PreSmooth,Smooth,MA_Method,VisualMode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,0,y);
      MDI[shift]  = iCustom(NULL,TimeFrame,IndicatorName,0,MathMode,Price,Length,ADX_Length,ADXR_Length,PreSmooth,Smooth,MA_Method,VisualMode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,1,y);
      
      ADX[shift]  = iCustom(NULL,TimeFrame,IndicatorName,0,MathMode,Price,Length,ADX_Length,ADXR_Length,PreSmooth,Smooth,MA_Method,VisualMode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,2,y);      
      ADXR[shift] = iCustom(NULL,TimeFrame,IndicatorName,0,MathMode,Price,Length,ADX_Length,ADXR_Length,PreSmooth,Smooth,MA_Method,VisualMode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,ShowArrows,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,3,y);
      }  
	
	return(0);
	}
	else _AllDMI(limit);

   return(0);
}


//-----
void _AllDMI(int limit)
{   
   double   sPDI, sMDI, sTR;
   
   for(int shift=limit;shift>=0;shift--) 
   {
      if(prevditime != Time[shift]) 
      {
      mMA[1]     = mMA[0]; 
      mLo[1]     = mLo[0]; 
      prevditime = Time[shift];
      }
   
      if(MathMode <  2) iprice[shift] = iMA(NULL,0,1,0,0,Price,shift);
      else 
      if(MathMode == 2) 
      {
      iprice[shift]  = iMA(NULL,0,1,0,0,2,shift);
      loprice[shift] = iMA(NULL,0,1,0,0,3,shift);
      mLo[0] = allAveragesOnArray(1,loprice,PreSmooth,MA_Method,masize,PreSmooth,shift);
      }       
   
   mMA[0] = allAveragesOnArray(0,iprice,PreSmooth,MA_Method,masize,PreSmooth,shift);       
      
      if(shift < Bars - PreSmooth)
      {
         if (MathMode == 0)
         {
         bulls[shift] = 0.5*(MathAbs(mMA[0] - mMA[1]) + (mMA[0] - mMA[1]));
         bears[shift] = 0.5*(MathAbs(mMA[0] - mMA[1]) - (mMA[0] - mMA[1]));
         }
         else  
         if (MathMode == 1)
         {
         double up = 0;      
         double dn = 10000000000;
            for(int i=0;i<Length;i++)
            {   
            up = MathMax(up,High[shift+i]);
            dn = MathMin(dn,Low [shift+i]);
            }
         bulls[shift] = mMA[0] - dn;
         bears[shift] = up - mMA[0];
         }
         else   
         if (MathMode == 2)
         {
         
         bulls[shift] = MathMax(0,0.5*(MathAbs(mMA[0] - mMA[1]) + (mMA[0] - mMA[1])));
         bears[shift] = MathMax(0,0.5*(MathAbs(mLo[1] - mLo[0]) + (mLo[1] - mLo[0])));
      
            if (bulls[shift] > bears[shift]) bears[shift] = 0;
            else 
            if (bulls[shift] < bears[shift]) bulls[shift] = 0;
            else
            if (bulls[shift] == bears[shift]) {bulls[shift] = 0; bears[shift] = 0;}
         }
      }
      
      if(MathMode == 1) int len = 1; else len = Length; 
     
      if(MathMode != 1) TR[shift] = MathMax(MathMax(High[shift] - Low[shift],High[shift] - Close[shift+1]),Close[shift+1] - Low[shift]);
      else TR[shift] = up - dn; 
      
   sPDI = allAveragesOnArray(2,bulls,len,MA_Method,masize,PreSmooth+len,shift);  
   sMDI = allAveragesOnArray(3,bears,len,MA_Method,masize,PreSmooth+len,shift);  
   sTR  = allAveragesOnArray(4,TR   ,len,MA_Method,masize,PreSmooth+len,shift);        
      
      if(sTR != 0)
      {
      rPDI[shift] = sPDI/sTR;
      rMDI[shift] = sMDI/sTR;
      double pDI  = 100*allAveragesOnArray(5,rPDI,Smooth,MA_Method,masize,PreSmooth+len+Smooth,shift); 
      double mDI  = 100*allAveragesOnArray(6,rMDI,Smooth,MA_Method,masize,PreSmooth+len+Smooth,shift);
      }
      else {pDI = 0; mDI = 0;}
      
     
      if(ADX_Length > 0)
      {
      if((pDI + mDI) != 0) DX[shift] = 100*MathAbs(pDI - mDI)/(pDI + mDI); else DX[shift] = 0;
            
      ADX[shift] = allAveragesOnArray(7,DX,ADX_Length,MA_Method,masize,PreSmooth+len+Smooth+ADX_Length,shift);
                 
      if(ADXR_Length > 0 && shift < Bars - len - ADXR_Length) ADXR[shift] = 0.5*(ADX[shift] + ADX[shift+ADXR_Length]);
      }
         
      if(VisualMode == 1)
      {
         if(pDI > mDI) {PDI[shift] = pDI - mDI; MDI[shift] = 0;} 
         else {MDI[shift] = pDI - mDI; PDI[shift] = 0;} 
      } 
      else {PDI[shift] = pDI; MDI[shift] = mDI;
    }
    trend[shift] = trend[shift+1];
    if (PDI[shift]>MDI[shift]) trend[shift] = 1;
    if (PDI[shift]<MDI[shift]) trend[shift] =-1;
   
    //
    //
    //
    //
    //
         
    if (ShowArrows)
    {
       deleteArrow(Time[shift]);
       if (trend[shift] != trend[shift+1])
       {
          if (trend[shift] == 1)  drawArrow(shift,arrowsUpColor,arrowsUpCode,false);
          if (trend[shift] ==-1)  drawArrow(shift,arrowsDnColor,arrowsDnCode, true);
       }
     }
     manageAlerts(); 
   }
   
}

//-----
string averageName(int mode,int& arraysize)
{   
   string ma_name = "";
   
   switch(mode)
   {
   case 1 : ma_name = "EMA"       ; break;
   case 2 : ma_name = "Wilder"    ; break;
   case 3 : ma_name = "LWMA"      ; break;
   case 4 : ma_name = "SineWMA"   ; break;
   case 5 : ma_name = "TriMA"     ; break;
   case 6 : ma_name = "LSMA"      ; break;
   case 7 : ma_name = "SMMA"      ; break;
   case 8 : ma_name = "HMA"       ; break;
   case 9 : ma_name = "ZeroLagEMA"; break;
   case 10: ma_name = "DEMA"      ; arraysize = 2; break;
   case 11: ma_name = "T3 basic"  ; arraysize = 6; break;
   case 12: ma_name = "InstTrend" ; break;
   case 13: ma_name = "Median"    ; break;
   case 14: ma_name = "GeoMean"   ; break;
   case 15: ma_name = "REMA"      ; break;
   case 16: ma_name = "ILRS"      ; break;
   case 17: ma_name = "IE/2"      ; break;
   case 18: ma_name = "TriMA_gen" ; break;
   case 19: ma_name = "VWMA"      ; break;
   case 20: ma_name = "JSmooth"   ; arraysize = 5; break;
   case 21: ma_name = "SMA_eq"    ; break;
   case 22: ma_name = "ALMA"      ; break;
   case 23: ma_name = "TEMA"      ; arraysize = 4; break;
   case 24: ma_name = "T3"        ; arraysize = 6; break;
   case 25: ma_name = "Laguerre"  ; arraysize = 4; break;
   default: ma_name = "SMA";      ; break;
   }
   
   return(ma_name);
   
}

double allAveragesOnArray(int index,double& price[],int period,int mode,int arraysize,int _start,int bar)
{
   double MA[3];  
        
    if(prevtime[index] != Time[bar])
    {
    ma[index][2]  = ma[index][1]; 
    ma[index][1]  = ma[index][0]; 
    for(int i=0;i<arraysize;i++) tmp[i][index][1] = tmp[i][index][0];
    
    prevtime[index] = Time[bar]; 
    }
   
   for(i=0;i<3;i++) MA[i] = ma[index][i];   
   
   switch(mode)
   {
   case 1 : ma[index][0] = EMAOnArray(price[bar],ma[index][1],period,bar);             break;
   case 2 : ma[index][0] = WilderOnArray(price[bar],ma[index][1],period,bar);          break;  
   case 3 : ma[index][0] = LWMAOnArray(price,period,bar);                              break;
   case 4 : ma[index][0] = SineWMAOnArray(price,period,bar);                           break;
   case 5 : ma[index][0] = TriMAOnArray(price,period,bar);                             break;
   case 6 : ma[index][0] = LSMAOnArray(price,period,bar);                              break;
   case 7 : ma[index][0] = SMMAOnArray(price,ma[index][1],period,_start,bar);          break;
   case 8 : ma[index][0] = HMAOnArray(price,period,_start,bar);                        break;
   case 9 : ma[index][0] = ZeroLagEMAOnArray(price,ma[index][1],period,_start,bar);    break;
   case 10: ma[index][0] = DEMAOnArray(index,0,price[bar],period,1,_start,bar);        break;
   case 11: ma[index][0] = T3_basicOnArray(index,0,price[bar],period,0.5,_start,bar);  break;
   case 12: ma[index][0] = ITrendOnArray(price,MA,period,_start,bar);                  break;
   case 13: ma[index][0] = MedianOnArray(price,period,bar);                            break;
   case 14: ma[index][0] = GeoMeanOnArray(price,period,_start,bar);                    break;
   case 15: ma[index][0] = REMAOnArray(price[bar],MA,period,0.5,_start,bar);           break;
   case 16: ma[index][0] = ILRSOnArray(price,period,bar);                              break;
   case 17: ma[index][0] = IE2OnArray(price,period,bar);                               break;
   case 18: ma[index][0] = TriMA_genOnArray(price,period,bar);                         break;
   case 19: ma[index][0] = VWMAOnArray(price,period,bar);                              break;
   case 20: ma[index][0] = JSmoothOnArray(index,0,price[bar],period,1,_start,bar);     break;
   case 21: ma[index][0] = SMA_eqOnArray(price,MA,period,_start,bar);                  break;
   case 22: ma[index][0] = ALMAOnArray(price,period,0.85,8,bar);                       break;
   case 23: ma[index][0] = TEMAOnArray(index,price[bar],period,1,_start,bar);          break;
   case 24: ma[index][0] = T3OnArray(index,0,price[bar],period,0.5,_start,bar);        break;
   case 25: ma[index][0] = LaguerreOnArray(index,price[bar],period,4,_start,bar);      break;
   default: ma[index][0] = SMAOnArray(price,period,bar);                       break;
   }
   
   return(ma[index][0]);
}



// MA_Method=0: SMA - Simple Moving Average
double SMAOnArray(double& array[],int per,int bar)
{
   double Sum = 0;
   for(int i = 0;i < per;i++) Sum += array[bar+i];
   
   return(Sum/per);
}
                           
// MA_Method=1: EMA - Exponential Moving Average
double EMAOnArray(double price,double prev,int per,int bar)
{
   if(bar >= Bars - 2) double ema = price;
   else 
   ema = prev + 2.0/(1+per)*(price - prev); 
   
   return(ema);
}

// MA_Method=2: Wilder - Wilder Exponential Moving Average
double WilderOnArray(double price,double prev,int per,int bar)
{
   if(bar >= Bars - 2) double wilder = price; //SMA(array1,per,bar);
   else 
   wilder = prev + (price - prev)/per; 
   
   return(wilder);
}

// MA_Method=3: LWMA - Linear Weighted Moving Average 
double LWMAOnArray(double& array[],int per,int bar)
{
   double Sum = 0;
   double Weight = 0;
   
      for(int i = 0;i < per;i++)
      { 
      Weight+= (per - i);
      Sum += array[bar+i]*(per - i);
      }
   if(Weight>0) double lwma = Sum/Weight;
   else lwma = 0; 
   return(lwma);
}

 
// MA_Method=4: SineWMA - Sine Weighted Moving Average
double SineWMAOnArray(double& array[],int per,int bar)
{
   double pi = 3.1415926535;
   double Sum = 0;
   double Weight = 0;
  
      for(int i = 0;i < per;i++)
      { 
      Weight+= MathSin(pi*(i+1)/(per+1));
      Sum += array[bar+i]*MathSin(pi*(i+1)/(per+1)); 
      }
   if(Weight>0) double swma = Sum/Weight;
   else swma = 0; 
   return(swma);
}

// MA_Method=5: TriMA - Triangular Moving Average
double TriMAOnArray(double& array[],int per,int bar)
{
   double sma;
   int len = MathCeil((per+1)*0.5);
   
   double sum=0;
   for(int i = 0;i < len;i++) 
   {
   sma = SMAOnArray(array,len,bar+i);
   sum += sma;
   } 
   double trima = sum/len;
   
   return(trima);
}

// MA_Method=6: LSMA - Least Square Moving Average (or EPMA, Linear Regression Line)
double LSMAOnArray(double& array[],int per,int bar)
{   
   double sum = 0;
   
   for(int i=per;i>=1;i--) sum += (i - (per + 1)/3.0)*array[bar+per-i];
   double lsma = 6.0*sum/(per*(per + 1.0));
   
   return(lsma);
}

// MA_Method=7: SMMA - Smoothed Moving Average
double SMMAOnArray(double& array[],double prev,int per,int _start,int bar)
{
   if(bar == Bars - _start) double smma = SMAOnArray(array,per,bar);
   else 
   if(bar < Bars - _start)
   {
   double Sum = 0;
   for(int i = 0;i < per;i++) Sum += array[bar+i+1];
   smma = (Sum - prev + array[bar])/per;
   }
   
   return(smma);
}
                                
// MA_Method=8: HMA - Hull Moving Average by Alan Hull
double HMAOnArray(double& array[],int per,int _start,int bar)
{
   if(per == 1) return(array[bar]);
   
   double temp[];
   int len = MathSqrt(per);
   
   ArrayResize(temp,len);
   
   if(bar == Bars - _start) double hma = array[bar]; 
   else
   if(bar < Bars - _start)
   {
   for(int i=0;i<len;i++) temp[i] = 2*LWMAOnArray(array,per/2,bar+i) - LWMAOnArray(array,per,bar+i); 
   hma = LWMAOnArray(temp,len,0); 
   }  

   return(hma);
}

// MA_Method=9: ZeroLagEMA - Zero-Lag Exponential Moving Average
double ZeroLagEMAOnArray(double& price[],double prev,int per,int _start,int bar)
{
   double alfa = 2.0/(1+per); 
   int lag = 0.5*(per - 1); 
   
   if(bar >= Bars - _start) double zema = price[bar];
   else 
   zema = alfa*(2*price[bar] - price[bar+lag]) + (1-alfa)*prev;
   
   return(zema);
}

// MA_Method=10: DEMA - Double Exponential Moving Average by Patrick Mulloy
double DEMAOnArray(int index,int num,double price,double per,double v,int _start,int bar)
{
   double _alpha = 2.0/(1+per);
   if(bar == Bars - _start) {double dema = price; tmp[num][index][0] = dema; tmp[num+1][index][0] = dema;}
   else 
   if(bar <  Bars - _start) 
   {
   tmp[num  ][index][0] = tmp[num  ][index][1] + _alpha*(price              - tmp[num  ][index][1]); 
   tmp[num+1][index][0] = tmp[num+1][index][1] + _alpha*(tmp[num][index][0] - tmp[num+1][index][1]); 
   dema                 = tmp[num  ][index][0]*(1+v) - tmp[num+1][index][0]*v;
   }
   
   return(dema);
}

// MA_Method=11: T3 by T.Tillson
double T3_basicOnArray(int index,int num,double price,int per,double v,int _start,int bar)
{
   double dema1, dema2;
   
   if(bar == Bars - 2) 
   {
   double T3 = price; 
   for(int k=0;k<6;k++) tmp[num+k][index][0] = price;
   }
   else 
   if(bar < Bars - 2) 
   {
   dema1 = DEMAOnArray(index,num  ,price,per,v,_start,bar); 
   dema2 = DEMAOnArray(index,num+2,dema1,per,v,_start,bar); 
   T3    = DEMAOnArray(index,num+4,dema2,per,v,_start,bar);
   }
   
   return(T3);
}

// MA_Method=12: ITrend - Instantaneous Trendline by J.Ehlers
double ITrendOnArray(double& price[],double& array[],int per,int _start,int bar)
{
   double alfa = 2.0/(per+1);
   if(bar < Bars - _start)
   double it = (alfa - 0.25*alfa*alfa)*price[bar] + 0.5*alfa*alfa*price[bar+1] - (alfa - 0.75*alfa*alfa)*price[bar+2] +
   2*(1-alfa)*array[1] - (1-alfa)*(1-alfa)*array[2];
   else
   it = (price[bar] + 2*price[bar+1] + price[bar+2])/4;
   
   return(it);
}
// MA_Method=13: Median - Moving Median
double MedianOnArray(double& price[],int per,int bar)
{
   double array[];
   ArrayResize(array,per);
   
   for(int i = 0; i < per;i++) array[i] = price[bar+i];
   ArraySort(array,WHOLE_ARRAY,0,MODE_DESCEND);
   
   int num = MathRound(0.5*(per - 1)); 
   if(MathMod(per,2) > 0) double median = array[num]; else median = 0.5*(array[num] + array[num+1]);
    
   return(median); 
}
// MA_Method=14: GeoMean - Geometric Mean
double GeoMeanOnArray(double& price[],int per,int _start,int bar)
{
   if(bar < Bars - _start)
   { 
   double gmean = MathPow(price[bar],1.0/per); 
   for(int i=1;i<per;i++) gmean *= MathPow(price[bar+i],1.0/per); 
   }
   
   return(gmean);
}
// MA_Method=15: REMA - Regularized EMA by Chris Satchwell 
double REMAOnArray(double price,double& array[],int per,double lambda,int _start,int bar)
{
   double _alpha =  2.0/(per + 1);
   if(bar >= Bars - _start) double rema = price;
   else 
   rema = (array[1]*(1 + 2*lambda) + _alpha*(price - array[1]) - lambda*array[2])/(1 + lambda); 
   
   return(rema);
}
// MA_Method=16: ILRS - Integral of Linear Regression Slope 
double ILRSOnArray(double& price[],int per,int bar)
{
   double sum = per*(per-1)*0.5;
   double sum2 = (per-1)*per*(2*per-1)/6.0;
     
   double sum1 = 0;
   double sumy = 0;
      for(int i=0;i<per;i++)
      { 
      sum1 += i*price[bar+i];
      sumy += price[bar+i];
      }
   double num1 = per*sum1 - sum*sumy;
   double num2 = sum*sum - per*sum2;
   
   if(num2 != 0) double slope = num1/num2; else slope = 0; 
   double ilrs = slope + SMAOnArray(price,per,bar);
   
   return(ilrs);
}
// MA_Method=17: IE/2 - Combination of LSMA and ILRS 
double IE2OnArray(double& price[],int per,int bar)
{
   double ie = 0.5*(ILRSOnArray(price,per,bar) + LSMAOnArray(price,per,bar));
      
   return(ie); 
}

// MA_Method=18: TriMAgen - Triangular Moving Average Generalized by J.Ehlers
double TriMA_genOnArray(double& array[],int per,int bar)
{
   int len1 = MathFloor((per + 1)*0.5);
   int len2 = MathCeil ((per + 1)*0.5);
   
   double sum=0;
   for(int i = 0;i < len2;i++) sum += SMAOnArray(array,len1,bar+i);
   double trimagen = sum/len2;
   
   return(trimagen);
}

// MA_Method=19: VWMA - Volume Weighted Moving Average 
double VWMAOnArray(double& array[],int per,int bar)
{
   double Sum = 0;
   double Weight = 0;
   
      for(int i = 0;i < per;i++)
      { 
      Weight+= Volume[bar+i];
      Sum += array[bar+i]*Volume[bar+i];
      }
   if(Weight>0) double vwma = Sum/Weight;
   else vwma = 0; 
   return(vwma);
} 
// MA_Method=20: JSmooth - Smoothing by Mark Jurik
double JSmoothOnArray(int index,int num,double price,int per,double pow,int _start,int bar)
{
   double beta   = 0.45*(per-1)/(0.45*(per-1)+2);
	double _alpha = MathPow(beta,pow);
	
	if(bar == Bars - _start) {tmp[num+4][index][0] = price; tmp[num+0][index][0] = price; tmp[num+2][index][0] = price;}
	else 
   if(bar <  Bars - _start) 
   {
	tmp[num+0][index][0] = (1 - _alpha)*price + _alpha*tmp[num+0][index][1];
	tmp[num+1][index][0] = (price - tmp[num+0][index][0])*(1-beta) + beta*tmp[num+1][index][1];
	tmp[num+2][index][0] = tmp[num+0][index][0] + tmp[num+1][index][0];
	tmp[num+3][index][0] = (tmp[num+2][index][0] - tmp[num+4][index][1])*MathPow((1 - _alpha),2) + MathPow(_alpha,2)*tmp[num+3][index][1];
	tmp[num+4][index][0] = tmp[num+4][index][1] + tmp[num+3][index][0]; 
   }
   return(tmp[num+4][index][0]);
}

// MA_Method=21: SMA_eq     - Simplified SMA
double SMA_eqOnArray(double& price[],double& array[],int per,int _start,int bar)
{
   if(bar == Bars - _start) double sma = SMAOnArray(price,per,bar);
   else 
   if(bar <  Bars - _start) sma = (price[bar] - price[bar+per])/per + array[1]; 
   
   return(sma);
}                 
// MA_Method=22: ALMA by Arnaud Legoux / Dimitris Kouzis-Loukas / Anthony Cascino
double ALMAOnArray(double& price[],int per,double offset,double sigma,int bar)
{
   double m = MathFloor(offset * (per - 1));
	double s = per/sigma;
		
	double w, sum =0, wsum = 0;		
	for (int i=0;i < per;i++) 
	{
	w = MathExp(-((i - m)*(i - m))/(2*s*s));
   wsum += w;
   sum += price[bar+(per-1-i)]*w; 
   }
   
   if(wsum != 0) double alma = sum/wsum; 
   
   return(alma);
}
  
// MA_Method=23: TEMA - Triple Exponential Moving Average by Patrick Mulloy
double TEMAOnArray(int index,double price,int per,double v,int _start,int bar)
{
   double _alpha = 2.0/(per+1);
	
	if(bar == Bars - _start) {tmp[0][index][0] = price; tmp[1][index][0] = price; tmp[2][index][0] = price;}
	else 
   if(bar <  Bars - _start) 
   {
	tmp[0][index][0] = tmp[0][index][1] + _alpha *(price     - tmp[0][index][1]);
	tmp[1][index][0] = tmp[1][index][1] + _alpha *(tmp[0][index][0] - tmp[1][index][1]);
	tmp[2][index][0] = tmp[2][index][1] + _alpha *(tmp[1][index][0] - tmp[2][index][1]);
	tmp[3][index][0] = tmp[0][index][0] + v*(tmp[0][index][0] + v*(tmp[0][index][0]-tmp[1][index][0]) - tmp[1][index][0] - v*(tmp[1][index][0] - tmp[2][index][0])); 
	}
   
   return(tmp[3][index][0]);
}
// MA_Method=24: T3 by T.Tillson (correct version) 
double T3OnArray(int index,int num,double price,int per,double v,int _start,int bar)
{
   double len = MathMax(2*(per + 2.0)/3.0 - 1,1), dema1, dema2;
   
   if(bar == Bars - 2) 
   {
   double T3 = price; 
   for(int k=0;k<6;k++) tmp[num+k][index][0] = T3;
   }
   else 
   if(bar < Bars - 2) 
   {
   dema1 = DEMAOnArray(index,num  ,price,len,v,_start,bar); 
   dema2 = DEMAOnArray(index,num+2,dema1,len,v,_start,bar); 
   T3    = DEMAOnArray(index,num+4,dema2,len,v,_start,bar);
   }
      
   return(T3);
}

// MA_Method=25: Laguerre filter by J.Ehlers
double LaguerreOnArray(int index,double price,int per,int order,int _start,int bar)
{
   double gamma = 1-10.0/(per+9);
   double aPrice[];
   
   ArrayResize(aPrice,order);
   
   for(int i=0;i<order;i++)
   {
      if(bar >= Bars - _start) tmp[i][index][0] = price;
      else
      {
         if(i == 0) tmp[i][index][0] = (1 - gamma)*price + gamma*tmp[i][index][1];
         else
         tmp[i][index][0] = -gamma * tmp[i-1][index][0] + tmp[i-1][index][1] + gamma * tmp[i][index][1];
      
      aPrice[i] = tmp[i][index][0];
      }
   }
   double laguerre = TriMA_genOnArray(aPrice,order,0);  

   return(laguerre);
}
//-----
string tf(int timeframe)
{
   switch(timeframe)
   {
   case PERIOD_M1:   return("M1");
   case PERIOD_M5:   return("M5");
   case PERIOD_M15:  return("M15");
   case PERIOD_M30:  return("M30");
   case PERIOD_H1:   return("H1");
   case PERIOD_H4:   return("H4");
   case PERIOD_D1:   return("D1");
   case PERIOD_W1:   return("W1");
   case PERIOD_MN1:  return("MN1");
   default:          return("N/A");
   }
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
            int forBar = 0;
      else      forBar = 1; 
      if (trend[forBar]!=trend[forBar+1])
      {
         if (trend[forBar]== 1) doAlert(0,"up");
         if (trend[forBar]==-1) doAlert(0,"down");
      }            
   }
}

//
//
//
//
//
 
void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," ",TF+" AllWildersDMI trend changed to ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"  AllWildersDMI"),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}